# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: search/grpc/search.proto
# Protobuf Python Version: 4.25.1
# pylint: skip-file
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x18search/grpc/search.proto\x12\rconfig.search\x1a\x1cgoogle/protobuf/struct.proto"{\n\x1b\x43reateIndexedContentRequest\x12\x10\n\x08keywords\x18\x01 \x03(\t\x12\r\n\x05title\x18\x02 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\x17\n\x0fitem_identifier\x18\x04 \x01(\t\x12\x11\n\titem_type\x18\x05 \x01(\t"d\n\x1c\x43reateIndexedContentResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"H\n\x1bSearchIndexedContentRequest\x12\x15\n\rsearch_phrase\x18\x01 \x01(\t\x12\x12\n\nitem_types\x18\x02 \x03(\t"d\n\x1cSearchIndexedContentResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct2\x85\x01\n\x1e\x43reateIndexedContentController\x12\x63\n\x06\x43reate\x12*.config.search.CreateIndexedContentRequest\x1a+.config.search.CreateIndexedContentResponse"\x00\x32\x85\x01\n\x1eSearchIndexedContentController\x12\x63\n\x06Search\x12*.config.search.SearchIndexedContentRequest\x1a+.config.search.SearchIndexedContentResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "search.grpc.search_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_CREATEINDEXEDCONTENTREQUEST"]._serialized_start = 73
    _globals["_CREATEINDEXEDCONTENTREQUEST"]._serialized_end = 196
    _globals["_CREATEINDEXEDCONTENTRESPONSE"]._serialized_start = 198
    _globals["_CREATEINDEXEDCONTENTRESPONSE"]._serialized_end = 298
    _globals["_SEARCHINDEXEDCONTENTREQUEST"]._serialized_start = 300
    _globals["_SEARCHINDEXEDCONTENTREQUEST"]._serialized_end = 372
    _globals["_SEARCHINDEXEDCONTENTRESPONSE"]._serialized_start = 374
    _globals["_SEARCHINDEXEDCONTENTRESPONSE"]._serialized_end = 474
    _globals["_CREATEINDEXEDCONTENTCONTROLLER"]._serialized_start = 477
    _globals["_CREATEINDEXEDCONTENTCONTROLLER"]._serialized_end = 610
    _globals["_SEARCHINDEXEDCONTENTCONTROLLER"]._serialized_start = 613
    _globals["_SEARCHINDEXEDCONTENTCONTROLLER"]._serialized_end = 746
# @@protoc_insertion_point(module_scope)



from enum import Enum

from .grpc_lib.search_pb2 import (
    CreateIndexedContentRequest,
    SearchIndexedContentRequest,
)
from .grpc_lib.search_pb2_grpc import (
    CreateIndexedContentControllerStub,
    SearchIndexedContentControllerStub,
)

SUCCESS_CODE = "0"


class IndexedContentRequestMessages:
    CREATE = CreateIndexedContentRequest()
    SEARCH = SearchIndexedContentRequest()


class IndexedContentRPCMethodNames:
    CREATE = "Create"
    SEARCH = "Search"


class IndexedContentControllerStubs:
    CREATE = CreateIndexedContentControllerStub()
    SEARCH = SearchIndexedContentControllerStub()


class ItemTypes(str, Enum):
    BLOG = "blog"
    COURSE = "course"
    MASTER_COURSE = "master_course"
    LESSON = "lesson"



from typing import ClassVar, List, Optional

from grpc.aio import insecure_channel
from pydantic import BaseModel, validate_call, UUID4

from .api_constants import (
    SUCCESS_CODE,
    IndexedContentControllerStubs,
    IndexedContentRequestMessages,
    IndexedContentRPCMethodNames,
    ItemTypes,
)


class ValidationError(Exception):
    pass


class ResponseCodeError(Exception):
    pass


class IndexedContent(BaseModel):
    """
    Configure the host and port of the search server.

    Parameters:
        host (str): The host address of the search server.
        port (int): The port number of the search server.
    """

    host: str = "127.0.0.1"
    port: str | int = "50051"

    __instance: ClassVar = None

    def __new__(cls, *args, **kwargs):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
        return cls.__instance

    async def perform_operation(
        self, rpc_method_name, request_message, controller_stub
    ):
        """Perform the operation.

        Parameters:
            rpc_method_name (str): The name of the RPC method (e.g., `Create`).
            request_message (gRPC message): The request message instance.
        """
        async with insecure_channel(f"{self.host}:{self.port}") as channel:
            controller_stub.channel = channel
            controller_stub.ready()
            RPCMethod = getattr(controller_stub, rpc_method_name, None)

            if RPCMethod:
                response = await RPCMethod(request_message)

                # Clear the request message for future use
                request_message.Clear()

                # Raise error if response is not successful
                if response.code != SUCCESS_CODE:
                    raise ResponseCodeError(response.code)

                return response
            else:
                raise ValidationError("Invalid RPC method name.")

    @validate_call
    async def create(
        self,
        title: str,
        content: str,
        item_identifier: str | UUID4,
        item_type: ItemTypes,
        keywords: Optional[List[str]] = None,
    ):
        """Create a new keyworded indexed content.
        Return:
            The search results in Darsman gRPC response format.
        """
        IndexedContentRequestMessages.CREATE.title = title
        IndexedContentRequestMessages.CREATE.content = content
        IndexedContentRequestMessages.CREATE.itemIdentifier = str(item_identifier)
        IndexedContentRequestMessages.CREATE.itemType = item_type.value
        if keywords:
            IndexedContentRequestMessages.CREATE.keywords.extend(keywords)

        return await self.perform_operation(
            IndexedContentRPCMethodNames.CREATE,
            IndexedContentRequestMessages.CREATE,
            IndexedContentControllerStubs.CREATE,
        )

    async def create_blog(
        self,
        title: str,
        content: str,
        identifier: str | UUID4,
        keywords: Optional[List[str]] = None,
    ):
        return await self.create(
            title=title,
            content=content,
            item_identifier=identifier,
            item_type=ItemTypes.BLOG,
            keywords=keywords,
        )

    async def create_course(
        self,
        title: str,
        content: str,
        identifier: str | UUID4,
        keywords: Optional[List[str]] = None,
    ):
        return await self.create(
            title=title,
            content=content,
            item_identifier=identifier,
            item_type=ItemTypes.COURSE,
            keywords=keywords,
        )

    async def create_master_course(
        self,
        title: str,
        content: str,
        identifier: str | UUID4,
        keywords: Optional[List[str]] = None,
    ):
        return await self.create(
            title=title,
            content=content,
            item_identifier=identifier,
            item_type=ItemTypes.MASTER_COURSE,
            keywords=keywords,
        )

    async def create_lesson(
        self,
        title: str,
        content: str,
        identifier: str | UUID4,
        keywords: Optional[List[str]] = None,
    ):
        return await self.create(
            title=title,
            content=content,
            item_identifier=identifier,
            item_type=ItemTypes.LESSON,
            keywords=keywords,
        )

    @validate_call
    async def search(self, search_phrase: str, item_types: List[ItemTypes]):
        """Search for a phrase across specified item types.

        Args:
            search_phrase (str): The phrase to search for.
            item_types (List[ItemTypes]): List of item types to filter the search.

        Return:
            The search results in Darsman gRPC response format.
        """
        # Set the search_phrase attribute in the request message
        IndexedContentRequestMessages.SEARCH.search_phrase = search_phrase
        IndexedContentRequestMessages.SEARCH.item_types.extend(item_types)

        # Perform the search operation
        return await self.perform_operation(
            IndexedContentRPCMethodNames.SEARCH,  # The name of the RPC method for searching
            IndexedContentRequestMessages.SEARCH,  # The request message for searching
            IndexedContentControllerStubs.SEARCH,  # The controller stub for searching
        )

    async def search_blog(self, search_phrase: str):
        return await self.search(
            search_phrase=search_phrase,
            item_types=[ItemTypes.BLOG],
        )

    async def search_course_lesson(self, search_phrase: str):
        return await self.search(
            search_phrase=search_phrase,
            item_types=[
                ItemTypes.COURSE,
                ItemTypes.LESSON,
                ItemTypes.MASTER_COURSE,
            ],
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
# pylint: skip-file
"""Client and server classes corresponding to protobuf-defined services."""

import grpc

from . import search_pb2 as search_dot_grpc_dot_search__pb2


class CreateIndexedContentControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            self.channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Create = self.channel.unary_unary(
                "/config.search.CreateIndexedContentController/Create",
                request_serializer=search_dot_grpc_dot_search__pb2.CreateIndexedContentRequest.SerializeToString,
                response_deserializer=search_dot_grpc_dot_search__pb2.CreateIndexedContentResponse.FromString,
            )


class CreateIndexedContentControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_CreateIndexedContentControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Create": grpc.unary_unary_rpc_method_handler(
            servicer.Create,
            request_deserializer=search_dot_grpc_dot_search__pb2.CreateIndexedContentRequest.FromString,
            response_serializer=search_dot_grpc_dot_search__pb2.CreateIndexedContentResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.search.CreateIndexedContentController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class CreateIndexedContentController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.search.CreateIndexedContentController/Create",
            search_dot_grpc_dot_search__pb2.CreateIndexedContentRequest.SerializeToString,
            search_dot_grpc_dot_search__pb2.CreateIndexedContentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class SearchIndexedContentControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            self.channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Search = self.channel.unary_unary(
                "/config.search.SearchIndexedContentController/Search",
                request_serializer=search_dot_grpc_dot_search__pb2.SearchIndexedContentRequest.SerializeToString,
                response_deserializer=search_dot_grpc_dot_search__pb2.SearchIndexedContentResponse.FromString,
            )


class SearchIndexedContentControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Search(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_SearchIndexedContentControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Search": grpc.unary_unary_rpc_method_handler(
            servicer.Search,
            request_deserializer=search_dot_grpc_dot_search__pb2.SearchIndexedContentRequest.FromString,
            response_serializer=search_dot_grpc_dot_search__pb2.SearchIndexedContentResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.search.SearchIndexedContentController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class SearchIndexedContentController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Search(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.search.SearchIndexedContentController/Search",
            search_dot_grpc_dot_search__pb2.SearchIndexedContentRequest.SerializeToString,
            search_dot_grpc_dot_search__pb2.SearchIndexedContentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )
